EXPORTS
 gtk_check_item_new
 gtk_check_item_new_with_label
 gtk_color_combo_find_color
 gtk_color_combo_get_color_at
 gtk_color_combo_get_type
 gtk_color_combo_new
 gtk_combobox_get_type
 gtk_dir_tree_get_type
 gtk_dir_tree_open_dir
 gtk_file_list_get_filename
 gtk_file_list_get_path
 gtk_file_list_get_type
 gtk_file_list_set_filter
 gtk_font_combo_get_type
 gtk_font_combo_new
 gtk_font_combo_select
 gtk_icon_file_selection_get_type
 gtk_icon_file_selection_new
 gtk_icon_list_add_from_data
 gtk_icon_list_get_link
 gtk_icon_list_get_type
 gtk_icon_list_new
 gtk_icon_list_remove
 gtk_icon_list_set_active_icon
 gtk_item_entry_get_type
 gtk_item_entry_set_justification
 gtk_plot_add_data
 gtk_plot_autoscale
 gtk_plot_axis_get_attributes
 gtk_plot_axis_get_visible
 gtk_plot_axis_hide_title
 gtk_plot_axis_labels_set_attributes
 gtk_plot_axis_labels_set_numbers
 gtk_plot_axis_set_attributes
 gtk_plot_axis_set_tick_labels
 gtk_plot_axis_set_ticks
 gtk_plot_axis_set_ticks_length
 gtk_plot_axis_set_ticks_limits
 gtk_plot_axis_set_ticks_width
 gtk_plot_axis_set_title
 gtk_plot_axis_set_visible
 gtk_plot_axis_show_labels
 gtk_plot_axis_show_ticks
 gtk_plot_axis_show_title
 gtk_plot_axis_title_set_attributes
 gtk_plot_axis_unset_ticks_limits
 gtk_plot_canvas_add_plot
 gtk_plot_canvas_cancel_action
 gtk_plot_canvas_export_ps
 gtk_plot_canvas_get_pixel
 gtk_plot_canvas_get_position
 gtk_plot_canvas_get_type
 gtk_plot_canvas_new
 gtk_plot_canvas_put_child
 gtk_plot_canvas_put_ellipse
 gtk_plot_canvas_put_line
 gtk_plot_canvas_put_rectangle
 gtk_plot_canvas_put_text
 gtk_plot_canvas_refresh
 gtk_plot_canvas_remove_child
 gtk_plot_canvas_set_active_plot
 gtk_plot_canvas_set_magnification
 gtk_plot_canvas_set_size
 gtk_plot_data_get_connector
 gtk_plot_data_get_line_attributes
 gtk_plot_data_get_symbol
 gtk_plot_data_hide
 gtk_plot_data_hide_legend
 gtk_plot_data_labels_set_attributes
 gtk_plot_data_new
 gtk_plot_data_set_connector
 gtk_plot_data_set_labels
 gtk_plot_data_set_legend
 gtk_plot_data_set_line_attributes
 gtk_plot_data_set_name
 gtk_plot_data_set_points
 gtk_plot_data_set_symbol
 gtk_plot_data_set_x_attributes
 gtk_plot_data_set_y_attributes
 gtk_plot_data_show_labels
 gtk_plot_data_show_xerrbars
 gtk_plot_data_show_yerrbars
 gtk_plot_get_axis
 gtk_plot_get_point
 gtk_plot_get_type
 gtk_plot_hide_legends
 gtk_plot_legends_move
 gtk_plot_legends_set_attributes
 gtk_plot_major_hgrid_set_attributes
 gtk_plot_major_vgrid_set_attributes
 gtk_plot_minor_hgrid_set_attributes
 gtk_plot_minor_vgrid_set_attributes
 gtk_plot_move
 gtk_plot_move_resize
 gtk_plot_new_with_size
 gtk_plot_remove_data
 gtk_plot_resize
 gtk_plot_set_legends_border
 gtk_plot_set_range
 gtk_plot_set_xrange
 gtk_plot_set_xscale
 gtk_plot_set_yrange
 gtk_plot_set_yscale
 gtk_plot_show_legends
 gtk_plot_x0_set_visible
 gtk_plot_x0line_set_attributes
 gtk_plot_y0_set_visible
 gtk_plot_y0line_set_attributes
 gtk_psfont_get_font
 gtk_sheet_add_column
 gtk_sheet_add_row
 gtk_sheet_cell_clear
 gtk_sheet_cell_get_text
 gtk_sheet_column_button_add_label
 gtk_sheet_column_set_justification
 gtk_sheet_delete_columns
 gtk_sheet_delete_rows
 gtk_sheet_freeze
 gtk_sheet_get_attributes
 gtk_sheet_get_link
 gtk_sheet_get_type
 gtk_sheet_insert_columns
 gtk_sheet_insert_rows
 gtk_sheet_link_cell
 gtk_sheet_new
 gtk_sheet_range_clear
 gtk_sheet_range_set_background
 gtk_sheet_remove_link
 gtk_sheet_row_button_add_label
 gtk_sheet_set_cell_text
 gtk_sheet_set_column_title
 gtk_sheet_set_column_width
 gtk_sheet_set_row_titles_width
 gtk_sheet_thaw
 gtk_sheet_get_rows_count
 gtk_sheet_get_columns_count
 gtk_icon_list_clear
 gtk_icon_list_set_selection_mode
 gtk_plot_canvas_set_background
 gtk_plot_data_set_numpoints
 gtk_plot_data_set_x
 gtk_plot_data_set_y
 gtk_plot_get_xrange
 gtk_plot_grids_set_visible
 gtk_plot_set_background
 gtk_border_combo_new
 gtk_sheet_attach
 gtk_sheet_button_attach
 gtk_sheet_change_entry
 gtk_sheet_clip_range
 gtk_sheet_get_cell_area
 gtk_sheet_get_entry
 gtk_sheet_hide_column_titles
 gtk_sheet_hide_row_titles
 gtk_sheet_range_set_border
 gtk_sheet_range_set_border_color
 gtk_sheet_range_set_editable
 gtk_sheet_range_set_font
 gtk_sheet_range_set_foreground
 gtk_sheet_range_set_justification
 gtk_sheet_row_button_justify
 gtk_sheet_row_set_sensitivity
 gtk_sheet_set_active_cell
 gtk_sheet_set_cell
 gtk_sheet_set_selection_mode
 gtk_sheet_show_column_titles
 gtk_sheet_show_row_titles
 gtk_sheet_unclip_range
 gtk_plot_add_function
 gtk_plot_canvas_child_new
 gtk_plot_canvas_get_active_plot
 gtk_plot_refresh
 gtk_plot_data_draw_points


