/* stylesheet for testgtksheet application */

@define-color sheet_default_bg_color lightgray;
@define-color sheet_default_fg_color black;

sheet { 
    background-color: @sheet_default_bg_color;
    color: @sheet_default_fg_color;
}

sheet .cell:selected {
    background-color: @theme_selected_bg_color;
    color: @theme_selected_fg_color;
}

/* sheet entry selection style */

sheet entry selection {
    background-color: @theme_selected_bg_color;
    color: @theme_selected_fg_color;
}

/* Note: "entry" selector is needed for entry types:
   GtkEntry, GtkDataEntry, GtkSpinButton
   */

sheet entry {
    background-color: @sheet_default_bg_color;
  min-height: 0px;
  padding: 0px;
}

/* Note: "text" selector is needed for entry types:
   GtkTextView, GtkDataTextView
   */ 

sheet.view text {
    background-color: @sheet_default_bg_color;
}

sheet.view text:backdrop {
    background-color: @sheet_default_bg_color;
}

/* eliminate column togglebutton warning */
sheet button.top {
  padding: 0px;
}

/* eliminate sheet entry spinbutton warning */
sheet spinbutton {
  padding: 0px;
  margin: 0px;
}

sheet spinbutton entry {
  min-height: 0px;
}

/* testgtksheet example1 - access via class */

@define-color testgtksheet_example1_default_bg lightyellow;

/* Note: :not(.left) excludes row title area */
sheet .example1class:not(.left) {
    /* light gridlines */
    border-color: black;
    border-width: 0.05px;
    border-image: none;
    border-style: solid;

    background-color: @testgtksheet_example1_default_bg;
    color: black;
}
sheet .example1class:not(.left):selected {
    background-color: @theme_selected_bg_color;
    color: @theme_selected_fg_color;
}
sheet .example1red:not(.left) {
    background-color: @testgtksheet_example1_default_bg;
    color: red;
    font: 28px Arial;
}
sheet .example1blue:not(.left) {
    background-color: @testgtksheet_example1_default_bg;
    color: blue;
    font: 36px Arial;
}
sheet .example1grey:not(.left) {
    background-color: darkgrey;
    color: green;
}

/* testgtksheet example2 - access via widget name */

@define-color testgtksheet_example2_bgcolor lightgray;

sheet#example2:not(.left) {
    /* heavier gridlines */
    border-color: black;
    border-width: 0.2px;
    border-image: none;
    border-style: solid;
}
sheet#example2 .blue {
    background-color: @testgtksheet_example2_bgcolor;
    color: blue;
}
sheet#example2 .red {
    background-color: @testgtksheet_example2_bgcolor;
    color: red;
}
sheet#example2 .black {
    background-color: @testgtksheet_example2_bgcolor;
    color: black;
}

/* testgtksheet example3 - access via widget name */

@define-color testgtksheet_example3_default_fg brown;

@define-color testgtksheet_example3_orange_bg orange;
@define-color testgtksheet_example3_orange_fg white;
@define-color testgtksheet_example3_lblue_bg lightblue;
@define-color testgtksheet_example3_lgreen_bg lightgreen;
@define-color testgtksheet_example3_red_bg red;
@define-color testgtksheet_example3_red_fg white;

sheet#example3 .orange {
    background-color: @testgtksheet_example3_orange_bg;
    color: @testgtksheet_example3_orange_fg;
}
sheet#example3 .orange text {
    background-color: @testgtksheet_example3_orange_bg;
    color: @testgtksheet_example3_orange_fg;
}
sheet#example3 .orange entry {
    background-color: @testgtksheet_example3_orange_bg;
    color: @testgtksheet_example3_orange_fg;
}
sheet#example3 .lblue {
    background-color: @testgtksheet_example3_lblue_bg;
    color: @testgtksheet_example3_default_fg;
}
sheet#example3 .lblue text {
    background-color: @testgtksheet_example3_lblue_bg;
    color: @testgtksheet_example3_default_fg;
}
sheet#example3 .lblue entry {
    background-color: @testgtksheet_example3_lblue_bg;
    color: @testgtksheet_example3_default_fg;
}
sheet#example3 .lgreen {
    background-color: @testgtksheet_example3_lgreen_bg;
    color: @testgtksheet_example3_default_fg;
}
sheet#example3 .red {
    background-color: @testgtksheet_example3_red_bg;
    color: @testgtksheet_example3_red_fg;
}

